/*
 * $Id: $
 *-
 *- memcheck.c
 *-
 *-   Purpose and Methods: (Destructively) Check a range or memory
 *-
 *-   Inputs  :
 *-   Outputs :
 *-   Controls:
 *-
 *-   Created  21-JAN-2014   John D. Hobbs
 *- $Revision: 1.3 $
 *-
*/

#include "xil_types.h"
#include "xparameters.h"
#include "memcheck.h"

static int verbose=0;

static int valueCheck( u32 *address, u32 nwords, u32 value) {
  u32 iw=0, status=0;
  u32 *curword = address;

  for( iw=0 ; iw<nwords ; iw++ ) {
	  if( verbose && ((iw%1000000)==0) ) xil_printf("        Filling word %d (address: 0x%08x)\n",iw,curword);
	  *curword++ = value;
  }
  if( verbose ) xil_printf("        Last address (fill) = 0x%08x\n",curword);
  curword = address;
  for( iw=0 ; iw<nwords ; iw++ ) if( *curword++ != value ) status += 1;
  return status;
}

int memcheck(u32 *address, u32 nwords) {
  u32 ibit=0,iw=0,status=0,ival=0;
  u32 *curword = address;

  xil_printf("Running memcheck for %d words with starting address = 0x%08x  (ends at 0x%08x)\n",nwords,address,(u32)(address+nwords));

  /* Write all zeros and check it */
  xil_printf("   zero check...\n");
  if( valueCheck(address,nwords,0) != 0 ) status |= 1;

  /* Write all ones and check it */
  xil_printf("   ones check...\n");
  if( valueCheck(address,nwords,0) != 0 ) status |= 2;

  /* Write on to each bit one after the other and check them*/
  xil_printf("   on bits check...\n");
  for( ibit=0 ; ibit<32 ; ibit++ ) {
	  ival = 1<<ibit;
	  xil_printf("       checking 0x%08x\n",ival);
	  if( valueCheck(address,nwords,ival) != 0 ) status |= 4;
  }

  /* Write a count and check it */
  xil_printf("   count check...\n");
  for( iw=0 ; iw<nwords ; iw++ ) *curword++ = iw;
  curword = address;
  for( iw=0 ; iw<nwords ; iw++ ) if( *curword++ != iw ) status |= 8;

  return status;
}

manual_memcheck() {
u32 *addr=(u32*)0xf, val, dir;
  while( addr != 0) {
	  xil_printf("Enter 1 for write, 0 for read: "); scanf("%d",&dir);
	  xil_printf("Enter address (0==>continue): "); scanf("%x",&addr);
	  if( dir ) {
		  xil_printf("Enter value: "); scanf("%x",&val);
		  *addr = val;
	  }
	  else {
		  val = *addr;
		  xil_printf("Value at 0x%08x = %x\n",addr,val);
	  }
  }
}

int auto_memcheck() {
  /* Run the basic memory check */
  u32 *start = (u32*)(DDR_START_ADDRESS + DDR_START_UNALLOC);
  u32 nwords32bits = (DDR_END_ADDRESS - (u32)start)/4;
  int status = memcheck(start,nwords32bits);
  xil_printf("Memory check return status = %d\n",status);
  return status;
}

